#!/usr/bin/env python
#
#######################
#
#    This program generates a line graph for the visualization of the 26G overlap data. It takes as input
#    any number of data columns. The input file uses a zero-based numbering system, so the first column is 0, the second is 1, etc.
#    The command line arguments are:
#
#    python 26G_plot_cmd.py [input folder] [input file] [labels file] [new folder] [x axis column] [y axis column 1] [y axis column 2] ...
#              
#
##################

### Import modules
import matplotlib.pyplot as plt
from pylab import *
import sys
import numpy as np
import math
import csv
import datetime
import os
from matplotlib import rcParams
import matplotlib.ticker as ticker
from operator import add

from matplotlib import rc

print sys.argv

def main():

    # Make the input and graph settings dictionaries
    input_settings, graph_settings = configure_settings()

    # Open the main graph data file and line labels
    line_data_text = import_data_from_file(input_settings['input_dir']+input_settings['input_file'])
    labels_data_text = import_data_from_file(input_settings['x_axis_labels_file'])

    # Process the main graph data file and line labels
    line_data = parse_data_from_lines(input_settings,line_data_text)
    series_labels = parse_data_to_list(labels_data_text)

    # Limit data to within range
    if graph_settings['xmin_val'] is not False or graph_settings['xmax_val'] is not False:
        line_data = subset_data(graph_settings,line_data)
    print line_data
    # Create output directory (if necessary) and prepare output file
    output_folder = create_output_folder(input_settings['output_dir'])

    # Write the data to an output file
    write_to_output(input_settings,graph_settings,line_data,series_labels)

    # Make the line graph
    make_line_graph(graph_settings,line_data,series_labels)

    # Save the line graphs to a file
    save_to_file(input_settings,graph_settings)

    # Display the graph, if selected
    if graph_settings['display_graph'] is True:
        plt.show()

    # Clear the image buffer before exiting
    plt.cla()
    plt.clf()
    plt.close()

    # Exit
    sys.exit(0)

def cm2inch(cm):
    return (cm/2.54)


def parse_data_to_list(data):

    # This function is used to convert a text list of elements to a python list
    # The input is the opened data, and the output is a list of items.

    new_list = []
    for line in data:
        new_list.append(line.strip('\n'))
    return new_list

def subset_data(graph_settings,line_data):

    # Uses the x_max and x_min values to take a subset of the data. This
    # prevents the lines from going past the edge of the figure when the axes
    # has x limits placed on it.

    if graph_settings['xmin_val'] is not False:
        x_min_index = line_data[0].index(str(graph_settings['xmin_val']))
    else:
        x_min = None

    if graph_settings['xmax_val'] is not False:
        x_max_index = line_data[0].index(str(graph_settings['xmax_val']))+1
    else:
        x_max = None

    new_line_data = []

    for i in range(len(line_data)):
        new_line_data.append([])
        new_line_data[i] = line_data[i][x_min_index:x_max_index]

    return new_line_data

def parse_data_from_lines(input_settings,datafile):


    # This functions parses the raw data into a format that can be easily
    # used, based on the columns indicated by the command line inputs. It
    # returns a list of lists

    try:
		line_data = [[]]
		for i in range(len(input_settings['value_cols'])):
		    line_data.append([])
		for line in datafile:
		    split_line = line.strip('\n').split('\t')
		    line_data[0].append(split_line[input_settings['x_label_col']])
		    for i in range(len(input_settings['value_cols'])):
		        if(split_line[input_settings['value_cols'][i]]) == '.':
		            line_data[i+1].append(0)
		        else:
		            line_data[i+1].append(split_line[input_settings['value_cols'][i]])
		return line_data
    except StandardError:
        print "There was an error parsing the datafile"


def create_output_folder(new_folder):

    # Checks to see if a directory exists, and if not, makes the directory. It
    # returns the directory path as output

    if not os.path.exists(new_folder):
        os.mkdir(new_folder)
    return new_folder


def configure_settings():

    # Initializes the file input and matplotlib settings. The graph settings
    # can be changed, but the input settings should not be. The function returns
    # the input and graph settings.

    ########## Do not change these!
    input_settings = {
        'input_dir':sys.argv[1],
        'input_file':sys.argv[2],
        'x_axis_labels_file':sys.argv[3],
        'new_folder':sys.argv[4],
        'x_label_col':int(sys.argv[5]),
        'value_cols':[]
        }

    for i in range(len(sys.argv)-6):
        #print 'create value columns - i: ', i, ' ',sys.argv[i+6]
        input_settings['value_cols'].append(int(sys.argv[i+6]))
    #print 'value_cols', input_settings['value_cols']
    if input_settings['new_folder'] == 'False':
        input_settings['output_dir'] = input_settings['input_dir']
    else:
        input_settings['output_dir'] = input_settings['input_dir']+input_settings['new_folder']+'_'+str(datetime.datetime.today().strftime("%d-%m-%Y_%H:%M:%S"))+'/'

    input_settings['output_file'] = input_settings['input_file']+'_LineGraphOutput'
    input_settings['output_image'] = input_settings['input_file']+'_LineGraph'

    ###########

    ########### These can be changed as needed to customize the graphs
    graph_settings = {
        'show_title':False,
        'show_xlabel':False,
        'show_ylabel':False,
        'show_legend':True,
        'show_legend_frame':False,
        'x_label':'Distance (nt)',
        'y_label':'Number of 26G',
        'file_label':input_settings['input_file'],
        'alpha_labels':True,
        'label_font':{'family':'sans serif','color':'black','weight':'normal','size':16},
        'title_font':{'family':'sans serif','color':'black','weight':'normal','size':10},
        'tick_font':{'family':'sans serif','color':'black','weight':'normal','size':8},
        'legend_font_size':8,
        'tick_label_color':'black',
        'show_x_tick_labels':True,
        'show_y_tick_labels':True,
        'x_tick_label_factor':1,
        'y_tick_label_factor':1,
        'export_svg':True,
        'export_png':False,
        'plot_height':4,
        'plot_width':4,
        'show_grid':False,
        'subplot_adjustments':{'bottom':0.3,'top':0.95,'left':0.3,'right':0.95,'wspace':0.05},
        'display_graph':False,
        'auto_colors':False,
        'line_colors':{0:'orange',1:'b',2:'r'},
        'line_width':1,
        'set_autoscale':False,
        'ymin_val':False,
        'ymax_val':False,
        'xmin_val':False,
        'xmax_val':False,
        'y_major_tick_locator':False,
        'y_minor_tick_locator':False,
        'x_major_tick_locator':10,
        'x_minor_tick_locator':5,
        'x_MaxNLocator':False,
        'y_MaxNLocator':2,
        'y_axis_bins':4,
        'bottom_spine':True,
        'top_spine':False,
        'left_spine':True,
        'right_spine':False,
        'headers':['Distance']
        }
    #############
    return (input_settings, graph_settings)


def import_data_from_file(filename):

    # This function opens a file from the disk. It returns the raw data.
    try:
        return open(filename)
    except IOError:
        print "Unable to open file: %s" %(filename)


def make_line_graph(graph_settings,line_data,series_labels):

    # Makes the line graph, using the data and the graph settings

    # Create the figure and axis
    fig, ax = plt.subplots()
    fig.set_size_inches(cm2inch(graph_settings['plot_width']),cm2inch(graph_settings['plot_height']))

    # Set ticks
    if graph_settings['x_major_tick_locator'] is not False:
        ax.xaxis.set_major_locator(ticker.MultipleLocator(graph_settings['x_major_tick_locator']))
    if graph_settings['x_minor_tick_locator'] is not False:
        ax.xaxis.set_minor_locator(ticker.MultipleLocator(graph_settings['x_minor_tick_locator']))
    if graph_settings['y_major_tick_locator'] is not False:
        ax.yaxis.set_major_locator(ticker.MultipleLocator(graph_settings['y_major_tick_locator']))
    if graph_settings['y_minor_tick_locator'] is not False:
        ax.yaxis.set_minor_locator(ticker.MultipleLocator(graph_settings['y_minor_tick_locator']))
    if graph_settings['x_MaxNLocator'] is not False:
        ax.yaxis.set_major_locator(ticker.MaxNLocator(graph_settings['x_MaxNLocator']))
    if graph_settings['y_MaxNLocator'] is not False:
        ax.yaxis.set_major_locator(ticker.MaxNLocator(graph_settings['y_MaxNLocator']))

    print "max(yticks()[0]) 1:", max(yticks()[0])
    print "ax.get_xlim(): ", ax.get_xlim()

    # Draw the lines
    for i in range(len(line_data)-1):
        if graph_settings['auto_colors']==True:
            plt.plot(line_data[0],line_data[i+1],
                label=str(series_labels[i]),
                lw=graph_settings['line_width'])
        else:
            plt.plot(line_data[0],line_data[i+1],
                label=str(series_labels[i]),
                lw=graph_settings['line_width'],
                color=graph_settings['line_colors'][i])

    # If selected, autoscale the graph. Else, using provided graph dimensions
    if graph_settings['set_autoscale'] is True:
        plt.autoscale()
    else:
        if graph_settings['xmin_val'] is not False:
            plt.xlim(xmin=graph_settings['xmin_val'])
        if graph_settings['ymin_val'] is not False:
            plt.ylim(ymin=graph_settings['ymin_val'])
        if graph_settings['xmax_val'] is not False:
            plt.xlim(xmax=graph_settings['xmax_val'])
        if graph_settings['ymax_val'] is not False:
            plt.ylim(ymax=graph_settings['ymax_val'])

    print "max(yticks()[0]) 3:", max(yticks()[0])
    #plt.locator_params(axis = 'y', nbins=graph_settings['y_axis_bins'])

    # Set visibility of spines
    ax.spines['bottom'].set_visible(graph_settings['bottom_spine'])
    ax.spines['top'].set_visible(graph_settings['top_spine'])
    ax.spines['left'].set_visible(graph_settings['left_spine'])
    ax.spines['right'].set_visible(graph_settings['right_spine'])

    # Set additional tick and grid parameters
    #plt.tick_params(axis='both', which='major', labelsize=graph_settings['tick_label_size'], labelcolor=graph_settings['tick_label_color'])


    # Draw the grid
    if graph_settings['show_grid'] is True:
        plt.grid(b=True, which='major',linestyle=':')
        plt.grid(b=True, which='minor',linestyle=':')

    ax.tick_params(which='both', direction='out',left='on', bottom='on', top='off',right='off')
    plt.tick_params(axis='y',which='major', direction='out',labelleft='on', labelright='off')
    # Set tick labels
    if graph_settings['show_x_tick_labels'] is True:
        ax.set_xticklabels([str(int(i*graph_settings['x_tick_label_factor'])) for i in xticks()[0]],size=8, visible=True, fontdict = graph_settings['tick_font'] )
    else:
        ax.set_xticklabels([])
    if graph_settings['show_y_tick_labels'] is True:
        ax.set_yticklabels([str(int(i*graph_settings['y_tick_label_factor'])) for i in yticks()[0]],size=8,visible=True, fontdict = graph_settings['tick_font'] )
    else:
        ax.set_yticklabels([])

    # Set initial margins around plot
    #left_margin = graph_settings['subplot_adjustments']['left']
    #right_margin = graph_settings['subplot_adjustments']['right']
    #bottom_margin = graph_settings['subplot_adjustments']['bottom']
    #top_margin = graph_settings['subplot_adjustments']['top']


    # Add title, legend and labels
    if graph_settings['show_title'] is True:
        title = graph_settings['file_label']
        plt.title(title,fontdict=graph_settings['title_font'])
        #top_margin=0.9
    if graph_settings['show_ylabel'] is True:
        plt.ylabel(graph_settings['y_label'], fontdict=graph_settings['label_font'])
        #left_margin=0.1
    if graph_settings['show_xlabel'] is True:
        plt.xlabel(graph_settings['x_label'], fontdict=graph_settings['label_font'])
        #bottom_margin=0.1
    if graph_settings['show_legend'] is True:
        plt.legend(fontsize=graph_settings['legend_font_size'],frameon=graph_settings['show_legend_frame'],ncol=1,loc=(.7,.6),handlelength=1,handletextpad=.1)
    ''''''
    #plt.subplots_adjust(left=left_margin,right=right_margin,bottom=bottom_margin,top=top_margin)
    plt.tight_layout(pad=0.25, w_pad=0.25, h_pad=0.25)

    return None


def write_to_output(input_settings,graph_settings, line_data, series_labels):

    # Save the line data to an output text file in the same directory as
    # the graphs

    # Create writer
    ofile  = open(input_settings['output_dir']+input_settings['output_file'], "wb")
    writer = csv.writer(ofile, delimiter='\t')

    # Add the headers
    headers = graph_settings['headers']
    for i in series_labels:
        headers.append(i)
    #print 'headers: ', headers
    writer.writerow(headers)
    #print 'data', line_data[0]
    # Write the data
    for i in range(len(line_data[0])):
        row = [line_data[0][i]]
        for j in range(len(line_data)-1):
            row.append(line_data[j+1][i])
        writer.writerow(row)
    writer.writerow('')

    # Close the file
    ofile.close()
    return

def save_to_file(input_settings, graph_settings):

    # This function saves the graphs to image files

    if graph_settings['export_svg'] is True:
        plt.savefig(input_settings['output_dir']+input_settings['output_image']+'.svg', format='svg')
    if graph_settings['export_png'] is True:
        plt.savefig(input_settings['output_dir']+input_settings['output_image']+'.png', format='png')
    #plt.savefig(input_settings['output_dir']+'_'+input_settings['output_image']+'.pdf', format='pdf')
    #plt.savefig(input_settings['output_dir']+'_'+input_settings['output_image']+'.agg', format='agg')
    #plt.savefig(input_settings['output_dir']+'_'+input_settings['output_image']+'.ps    ', format='ps')
    return


if __name__ == "__main__":
    sys.exit(main())

