# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
#######################
#
#    This program reformats a specific Galaxy output into a table format. Its use should be paired with
#    the program offset_preformatted_cmd.py, which the data in the correct format.
#    The data will be formatted into a table, with the first row being the list of line IDs. The
#    first column (after the first row) will be the x axis labels. Each other column will
#    contain the data for a line on the graph. 
#
#	python offset_data_reformat.py [options]
#	Options:
#	--in_dir	Input directory
#	--in_file	Input file
#	--line_ID	ID of the lines on this graph
#	--length	Length of each index
#	--index_col	Column of index values
#	--y_col		Column of y values
#	--out_dir	Output folder
#
##################


### Import modules
import sys
import numpy as np
import math
import csv
import os
from operator import add
import argparse

parser = argparse.ArgumentParser(description='Reformat the data from offset chart workflow')
parser.add_argument('--in_dir',   type=str, help='Input directory')
parser.add_argument('--in_file',  type=str, help='Input file')
parser.add_argument('--line_ID', type=str, nargs='+', help='ID of the lines on this graph')
parser.add_argument('--length',   type=int, help='Length of each index')
parser.add_argument('--index_col',type=int, help='Column of index values')
parser.add_argument('--y_col',    type=int, help='Column of y values')
parser.add_argument('--out_dir',  type=str, help='Output folder')

def get_args():
    '''
    Return the argparse arguments as a dictionary

    '''
    args = vars(parser.parse_args())
    return args

def main():

    # Get the command line arguments
    args = get_args()

    # Set the configuration settings dictionary
    input_settings = configure_settings(args)

    # Open the main graph data file and line labels
    raw_line_data = import_data_from_file(input_settings['input_dir']+input_settings['input_file'])

    # Process the main graph data file and line labels
    line_data = parse_data_from_lines(input_settings,raw_line_data)

    #print 'line_data', line_data
    # Create output directory (if necessary) and prepare output file
    output_folder = create_output_folder(input_settings['output_dir'])

    # Write the data to an output file
    write_to_output(input_settings,line_data)

    # Exit
    sys.exit(0)


def parse_data_from_lines(input_settings,datafile):

    # This functions parses the raw data into a format that can be easily
    # used, based on the columns indicated by the command line inputs. It
    # returns a list of lists


    # Create the lists for the output data
    output_data = [[]]
    for i in range(len(input_settings['line_ID'])):
        output_data.append([])
    # Setup the index counters
    range_index = 0 # The index of the data in the range
    data_index = 1 #The index of the lists in output_data

    # Copy these values from input settings to make the next section easier to read
    index_col = input_settings['index_col']
    line_ID = input_settings['line_ID']
    value_col = input_settings['value_col']

    # For each line in the input ...
    for line in datafile:
        # Split the line by tabs and remove ending return character
        split_line = line.strip('\n').split('\t')

        # If its the first set of values, create the x axis index
        if data_index is 1:
            output_data[0].append(split_line[index_col][len(line_ID[data_index-1]):])
        # If the first 2 characters in the first column match the line ID, append the data
        if split_line[index_col][:len(line_ID[data_index-1])] == line_ID[data_index-1]:
            output_data[data_index].append(split_line[value_col])

        # Increment the range index
        range_index += 1
        # If the range index reaches the max, reset the counters and start the next line ID
        if range_index is input_settings['range_size']:
            range_index = 0
            data_index += 1

    return output_data


def create_output_folder(new_folder):

    # Checks to see if a directory exists, and if not, makes the directory. It
    # returns the directory path as output

    if not os.path.exists(new_folder):
        os.mkdir(new_folder)
    return new_folder

def configure_settings(args):

    # Initializes the file input and matplotlib settings. The graph settings
    # can be changed, but the input settings should not be. The function returns
    # the input and graph settings.

    print args

    ########## Do not change these!
    input_settings = {
        'input_dir':args['in_dir'],
        'input_file':args['in_file'],
        'output_dir':args['out_dir'],
        'line_ID':args['line_ID'],
        'index_col':args['index_col'],
        'value_col':args['y_col'],
        'range_size':args['length']
        }

    if input_settings['output_dir'] is None:
        input_settings['output_dir'] = input_settings['input_dir']

    return input_settings


def import_data_from_file(filename):

    # This function opens a file from the disk. It returns the raw data.
    try:
        return open(filename)
    except IOError:
        print "Unable to open file: %s" %(filename)


def write_to_output(input_settings, data):

    # Save the line data to an output text file in the same directory as
    # the graphs

    # Create writer
    ofile  = open(input_settings['output_dir']+input_settings['input_file']+'_Formatted.txt', "wb")
    writer = csv.writer(ofile, delimiter='\t')

    # Add the headers
    writer.writerow(input_settings['line_ID'])

    # Write the data
    for i in range(len(data[0])):
        row = []
        for j in range(len(data)):
            row.append(data[j][i])
        writer.writerow(row)

    # Close the file
    ofile.close()
    return


if __name__ == "__main__":
    sys.exit(main())
