# -*- coding: utf-8 -*-
#!/usr/bin/env python
#
#######################
#
#    This program generates a scatter plot of numeric data. One x-axis column and
#    an unlimited number of data columns can be used. The input file uses a zero-based numbering
#    system, so the first column is 0, the second is 1, etc.
#    The command line syntax is:
#
#        python scatter_plot_cmd.py [input folder] [input file] [new folder] [group col] [x col] [y col] [intensity col]
#
#
##################

### Import modules
import matplotlib.pyplot as plt
from pylab import *
import sys
import numpy as np
from scipy import stats
import math
import csv
import datetime
import os
from matplotlib import rcParams
import matplotlib.ticker as ticker
from operator import add
import matplotlib.font_manager
import argparse

print sys.argv

parser = argparse.ArgumentParser(description='Take inputs for creating scatter plots')
parser.add_argument('--in_dir',   type=str, help='Input Directory')
parser.add_argument('--in_file',  type=str, help='Input File')
parser.add_argument('--group_col', type=int,help='Column ID for scatter grouping')
parser.add_argument('--x_col',   type=int, help='Column ID for x values')
parser.add_argument('--y_col',type=int, help='Column ID for y values')
parser.add_argument('--value_col',    type=int, help='Column of scatter point values')
parser.add_argument('--new_folder',  type=str, help='Create a new folder?')
parser.add_argument('--color',  type=str, help='Color of the dots?')

def main():

    # Get the command line arguments
    args = get_args()

    # Make the input and graph settings dictionaries
    input_settings, graph_settings = configure_settings(args)

    # Open the main graph data file and line labels
    line_data_text = import_data_from_file(input_settings['input_dir']+input_settings['input_file'])
    #labels_data_text = import_data_from_file(input_settings['x_axis_labels_file'])

    # Process the main graph data file and line labels
    graph_data = parse_data_from_lines(input_settings,graph_settings,line_data_text)
    #series_labels = parse_data_to_list(labels_data_text)

    # Create output directory (if necessary) and prepare output file
    output_folder = create_output_folder(input_settings['output_dir'])

    # Write the data to an output file
    write_to_output(input_settings,graph_settings,graph_data)

    # Make the line graph
    for key in graph_data.keys():
        make_line_graph(graph_settings,key, graph_data[key])
        # Save the line graphs to a file
        save_to_file(input_settings,graph_settings)
        # Display the graph, if selected
        if graph_settings['display_graph'] is True:
            plt.show()
        # Clear the image buffer before exiting
        plt.cla()
        plt.clf()
        plt.close()

    # Exit
    sys.exit(0)

def parse_data_to_list(data):

    # This function is used to convert a text list of elements to a python list
    # The input is the opened data, and the output is a list of items.

    new_list = []
    for line in data:
        new_list.append(line.strip('\n'))
    return new_list

def get_args():
    '''`
    Return the argparse arguments as a dictionary

    '''
    args = vars(parser.parse_args())
    return args


def parse_data_from_lines(input_settings,graph_settings,datafile):

    # This functions parses the raw data into a format that can be easily
    # used, based on the columns indicated by the command line inputs. It
    # returns a dict of groups of lists
    data_dict= dict()

    for line in datafile:
        split_line = line.strip('\n').split('\t')

        if input_settings['group_col'] == None:
            group = 0
        else:
            group = split_line[input_settings['group_col']]

        if group not in data_dict.keys():
            data_dict[group] = [[],[],[],[]]

        data_dict[group][0].append(int(split_line[input_settings['x_col']]))
        data_dict[group][1].append(int(split_line[input_settings['y_col']]))
        data_dict[group][3].append(str(split_line[graph_settings['label_col']]))

        if input_settings['val_col'] == None:
            data_dict[group][2].append(graph_settings['default_intensity_val'])
        else:
            data_dict[group][2].append(int(split_line[input_settings['val_col']]))

    return data_dict

def create_output_folder(new_folder):

    # Checks to see if a directory exists, and if not, makes the directory. It
    # returns the directory path as output

    if not os.path.exists(new_folder):
        os.mkdir(new_folder)
    return new_folder


def configure_settings(args):

    # Initializes the file input and matplotlib settings. The graph settings
    # can be changed, but the input settings should not be. The function returns
    # the input and graph settings.


    ########## Do not change these!
    input_settings = {
	'input_dir':args['in_dir'],
	'input_file':args['in_file'],
        'new_folder':args['new_folder'],
        'group_col':args['group_col'],
        'x_col':args['x_col'],
        'y_col':args['y_col'],
        'val_col':args['value_col'],
        'color':args['color'],
        }

    if input_settings['new_folder'] == None:
        input_settings['output_dir'] = input_settings['input_dir']
    else:
        input_settings['output_dir'] = input_settings['input_dir']+input_settings['new_folder']+'/'#'_'+str(datetime.datetime.today().strftime("%d-%m-%Y_%H:%M:%S"))+'/'

    input_settings['output_file'] = input_settings['input_file']+'_ScatterOutput_'+str(input_settings['x_col'])+'_x_'+str(input_settings['y_col'])
    input_settings['output_image'] = input_settings['input_file']+'_ScatterPlot_'+str(input_settings['x_col'])+'_x_'+str(input_settings['y_col'])
    if input_settings['color'] == None:
        input_settings['color'] = 'black'


    ###########

    ########### These can be changed as needed to customize the graphs
    graph_settings = {
        'show_title':False,
        'show_xlabel':False,
        'show_ylabel':False,
        'show_legend':False,
        'x_label':'X',
        'y_label':'Y',
        'file_label':input_settings['input_file'],
        'alpha_labels':True,
        'label_font':{'family':'sans serif','color':'black','weight':'normal','size':16},
        'title_font':{'family':'sans serif','color':'black','weight':'normal','size':10},
        'tick_font':{'family':'sans serif','color':'black','weight':'normal','size':24},
        'tick_label_size':14,
        'tick_label_color':'black',
        'export_svg':False,
        'export_png':False,
        'export_tif':True,
        'export_eps':True,
        'marker_color':input_settings['color'],
        'label_col':0,
        'stats':True,
        'plot_height':5,
        'plot_width':5,
        'show_grid':False,
        'subplot_adjustments':{'bottom':0.18,'top':0.95,'left':0.25,'right':0.95,'wspace':0.1},
        #'subplot_adjustments':{'bottom':0.1,'top':0.8,'left':0.05,'right':0.98,'wspace':0.1},
        'display_graph':False,
        'set_autoscale':'find_range',
        'ymin_val':0,
        'ymax_val':1500,
        'xmin_val':0,
        'xmax_val':1500,
        'show_x_tick_labels':False,
        'show_y_tick_labels':False,
        'x_major_tick_locator':False,
        'x_minor_tick_locator':False,
        'y_major_tick_locator':False,
        'y_minor_tick_locator':False,
        'x_tick_label_factor':1,
        'y_tick_label_factor':1,
        'x_tick_label_padding':5,
        'y_tick_label_padding':5,
        'x_MaxNLocator':2,
        'y_MaxNLocator':2,
        'y_axis_bins':2,
        'x_axis_bins':2,
        'default_margins':{'left_margin':0.025,'right_margin':0.975,'bottom_margin':0.025,'top_margin':0.975},
        'show_spine':{'bottom':True,'top':False,'left':True,'right':False},
        'spine_width':{'bottom':2.0,'top':2.0,'left':2.0,'right':2.0},
        'tick_width':2,
        'tick_length':8,
        'marker_shape':'o',
        'default_intensity_val':10,
        'headers':['Label','Group','X','Y','Val']
        }
    #############
    return (input_settings, graph_settings)


def import_data_from_file(filename):

    # This function opens a file from the disk. It returns the raw data.
    try:
        return open(filename)
    except IOError:
        print "Unable to open file: %s" %(filename)

def cm2inch(cm):
    return (cm/2.54)

def find_range(x,y):
    range_values = (min(x),max(x),min(y),max(y))
    return range_values

def roundup(x,y):
    ### For any number x, this function rounds up to the nearest 0th place, times y.
    ### Example: roundup(2385,5) rounds to 5000. roundup(25,4) rounds to 40
    y = 10**int(math.log10(x))*y
    if y == 10000:
        y = 1000
    return x if x % y == 0 else x + y - x % y

def make_line_graph(graph_settings,group,data):

    # Makes the line graph, using the data and the graph settings

    # Set initial margins around plot
    #left_margin = graph_settings['default_margins']['left_margin']
    #right_margin = graph_settings['default_margins']['right_margin']
    #bottom_margin = graph_settings['default_margins']['bottom_margin']
    #top_margin = graph_settings['default_margins']['top_margin']

    # Create the figure and axis
    fig, ax = plt.subplots()
    fig.set_size_inches(cm2inch(graph_settings['plot_width']),cm2inch(graph_settings['plot_height']))

    # Set figure size
    fig.set_size_inches(cm2inch(graph_settings['plot_width']),cm2inch(graph_settings['plot_height']))

    # Set ticks
    if graph_settings['x_major_tick_locator'] is not False:
        ax.xaxis.set_major_locator(ticker.MultipleLocator(graph_settings['x_major_tick_locator']))
    if graph_settings['x_minor_tick_locator'] is not False:
        ax.xaxis.set_minor_locator(ticker.MultipleLocator(graph_settings['x_minor_tick_locator']))
    if graph_settings['y_major_tick_locator'] is not False:
        ax.yaxis.set_major_locator(ticker.MultipleLocator(graph_settings['y_major_tick_locator']))
    if graph_settings['y_minor_tick_locator'] is not False:
        ax.yaxis.set_minor_locator(ticker.MultipleLocator(graph_settings['y_minor_tick_locator']))
    if graph_settings['x_MaxNLocator'] is not False:
        ax.yaxis.set_major_locator(ticker.MaxNLocator(graph_settings['x_MaxNLocator']))
    if graph_settings['y_MaxNLocator'] is not False:
        ax.yaxis.set_major_locator(ticker.MaxNLocator(graph_settings['y_MaxNLocator']))

    # Draw the graph
    scatterplot = plt.scatter(data[0],data[1], s=data[2], marker=graph_settings['marker_shape'],c=graph_settings['marker_color'],linewidth=0,vmin=0,vmax=0)

    # If selected, autoscale the graph. Else, using provided graph dimensions
    if graph_settings['set_autoscale'] is 'autoscale':
        plt.autoscale()
    elif graph_settings['set_autoscale'] is 'fixed':
        if graph_settings['xmin_val'] is not False:
            plt.xlim(xmin=graph_settings['xmin_val'])
        if graph_settings['ymin_val'] is not False:
            plt.ylim(ymin=graph_settings['ymin_val'])
        if graph_settings['xmax_val'] is not False:
            plt.xlim(xmax=graph_settings['xmax_val'])
        if graph_settings['ymax_val'] is not False:
            plt.ylim(ymax=graph_settings['ymax_val'])
    elif graph_settings['set_autoscale'] is 'find_range':
            range_values = find_range(data[0],data[1])
            plt.xlim(xmin=(range_values[0]*1.1))
            plt.xlim(xmax=(roundup(range_values[1],1)))
            print 'xmax:', roundup(range_values[1],1)
            plt.ylim(ymin=(range_values[2]*1.1))
            plt.ylim(ymax=(roundup(range_values[3],1)))
            print 'ymax:', roundup(range_values[3],1)
            ax.xaxis.set_major_locator(ticker.MultipleLocator(roundup(range_values[1],1)))
            #ax.xaxis.set_minor_locator(ticker.MultipleLocator(roundup(range_values[1]/2,graph_settings['x_major_tick_locator'])))
            ax.yaxis.set_major_locator(ticker.MultipleLocator(roundup(range_values[3],1)))
            #ax.yaxis.set_minor_locator(ticker.MultipleLocator(roundup(range_values[3]/2,graph_settings['y_major_tick_locator'])))

    # Set additional tick and grid parameters
    #ax.tick_params(axis='both', which='major', labelsize=graph_settings['tick_label_size'], labelcolor=graph_settings['tick_label_color'])
    ax.tick_params(which='both', direction='out',left='on', bottom='on', top='off',right='off', width=graph_settings['tick_width'], length=graph_settings['tick_length'])
    ax.xaxis.labelpad = graph_settings['x_tick_label_padding']
    ax.yaxis.labelpad = graph_settings['y_tick_label_padding']

    # Draw the grid
    if graph_settings['show_grid'] is True:
        plt.grid(b=True, which='major',linestyle=':')
        plt.grid(b=True, which='minor',linestyle=':')
    # Set tick labels
    if graph_settings['show_x_tick_labels'] is True:
        ax.set_xticklabels([str(int(i*graph_settings['x_tick_label_factor'])) for i in xticks()[0]],size=10, visible=True, fontdict = graph_settings['tick_font'] )
    else:
        ax.set_xticklabels([])
    if graph_settings['show_y_tick_labels'] is True:
        ax.set_yticklabels([str(int(i*graph_settings['y_tick_label_factor'])) for i in yticks()[0]],size=10,visible=True, fontdict = graph_settings['tick_font'])
    else:
        ax.set_yticklabels([])

    # Set visibility of spines
    ax.spines['bottom'].set_visible(graph_settings['show_spine']['bottom'])
    ax.spines['top'].set_visible(graph_settings['show_spine']['top'])
    ax.spines['left'].set_visible(graph_settings['show_spine']['left'])
    ax.spines['right'].set_visible(graph_settings['show_spine']['right'])
    ax.spines['bottom'].set_linewidth(graph_settings['spine_width']['bottom'])
    ax.spines['top'].set_linewidth(graph_settings['spine_width']['top'])
    ax.spines['left'].set_linewidth(graph_settings['spine_width']['left'])
    ax.spines['right'].set_linewidth(graph_settings['spine_width']['right'])




    # Add title, legend and labels
    title = graph_settings['file_label']+' - '+ str(group)
    if graph_settings['show_title'] is True:
        plt.title(title,fontdict=graph_settings['title_font'])
        #top_margin=0.9
    if graph_settings['show_ylabel'] is True:
        plt.ylabel(graph_settings['y_label'], fontdict=graph_settings['label_font'])
        #left_margin=0.1
    if graph_settings['show_xlabel'] is True:
        plt.xlabel(graph_settings['x_label'], fontdict=graph_settings['label_font'])
        #bottom_margin=0.1
    if graph_settings['show_legend'] is True:
        plt.legend()

    #plt.subplots_adjust(left=left_margin,right=right_margin,bottom=bottom_margin,top=top_margin)


    ax.tick_params(axis='x', which='major', pad=graph_settings['x_tick_label_padding'])
    ax.tick_params(axis='y', which='major', pad=graph_settings['y_tick_label_padding'])
    plt.tight_layout()

    #ax.xaxis.labelpad = graph_settings['x_tick_label_padding']
    #ax.yaxis.labelpad = graph_settings['y_tick_label_padding']

    return

def calculate_stats(x,y):

    statistics = dict()

    #for a in range(len(x)):
    #    print "x - ", x[a], " ", "y - ", y[a]

    statistics['slope'], statistics['intercept'], statistics['r_value'], statistics['p_value'], statistics['std_err'] = stats.linregress(x,y)
    statistics['r_squared'] = statistics['r_value']**2
    #print "slope = ", statistics['slope']
    #print "intercept = ", statistics['intercept']
    #print "r_value = ", statistics['r_value']
    #print "p_value = ", statistics['p_value']
    #print "std_err = ", statistics['std_err']
    #print "r_squared = ", statistics['r_squared']

    return statistics

def write_to_output(input_settings,graph_settings, graph_data):

    # Save the line data to an output text file in the same directory as
    # the graphs

    # Create writer
    ofile  = open(input_settings['output_dir']+input_settings['output_file'], "wb")
    writer = csv.writer(ofile, delimiter='\t')

    # Write the data
    for group in graph_data.keys():
        if graph_settings['stats'] == True:
            statistics = calculate_stats(graph_data[group][0],graph_data[group][1])
            writer.writerow(["Statistics:"])
            writer.writerow(("slope = ", statistics['slope']))
            writer.writerow(("intercept = ", statistics['intercept']))
            writer.writerow(("r_value = ", statistics['r_value']))
            #writer.writerow(("p_value = ", statistics['p_value']))
            writer.writerow(("std_err = ", statistics['std_err']))
            writer.writerow(("r_squared = ", statistics['r_squared']))
            writer.writerow('')
            ranges = find_range(graph_data[group][0],graph_data[group][1])
            writer.writerow(["Ranges:"])
            writer.writerow(("X Min = ", ranges[0]))
            writer.writerow(("X Max = ", ranges[1]))
            writer.writerow(("Y Min = ", ranges[2]))
            writer.writerow(("Y Max = ", ranges[3]))
            writer.writerow('')

        writer.writerow(graph_settings['headers'])
        for point in range(len(graph_data[group][0])):
            writer.writerow((graph_data[group][3][point],group,graph_data[group][0][point],
                            graph_data[group][1][point],
                            graph_data[group][2][point]
                            ))
        writer.writerow('')

    # Close the file
    ofile.close()
    return

def save_to_file(input_settings, graph_settings):

    # This function saves the graphs to image files

    if graph_settings['export_svg'] is True:
        plt.savefig(input_settings['output_dir']+input_settings['output_image']+'.svg', format='svg')
    if graph_settings['export_png'] is True:
        plt.savefig(input_settings['output_dir']+input_settings['output_image']+'.png', format='png')
    if graph_settings['export_tif'] is True:
        plt.savefig(input_settings['output_dir']+input_settings['output_image']+'.tif', format='tif')
    if graph_settings['export_eps'] is True:
        plt.savefig(input_settings['output_dir']+input_settings['output_image']+'.eps', format='eps')
    
    return


if __name__ == "__main__":
    sys.exit(main())

